@@ -7,14 +7,17 @@
 #include <glib-object.h>
 
 #include <clutter/clutter.h>
-#include <clutter/x11/clutter-x11.h>
 #include <meta/common.h>
 #include <meta/display.h>
-#include <meta/main.h>
+#include <meta/keybindings.h>
+#include <meta/meta-context.h>
 #include <meta/util.h>
 #include <meta/meta-window-group.h>
 
+#include <systemd/sd-daemon.h>
+
 #include "kiosk-backgrounds.h"
+#include "kiosk-gobject-utils.h"
 #include "kiosk-input-sources-manager.h"
 #include "kiosk-service.h"
 
@@ -26,6 +29,7 @@ struct _KioskCompositor
 
         /* weak references */
         MetaDisplay *display;
+        MetaContext *context;
         ClutterBackend *backend;
         ClutterActor *stage;
 
@@ -58,6 +62,7 @@ kiosk_compositor_dispose (GObject *object)
         }
 
         g_clear_weak_pointer (&self->stage);
+        g_clear_weak_pointer (&self->context);
         g_clear_weak_pointer (&self->display);
         g_clear_weak_pointer (&self->backend);
 
@@ -112,12 +117,95 @@ register_with_display_manager (KioskCompositor *self)
         }
 }
 
+static void
+register_with_systemd (KioskCompositor *self)
+{
+        sd_notify (TRUE, "READY=1");
+}
+
 static void
 register_session (KioskCompositor *self)
 {
-        meta_register_with_session ();
+        meta_context_notify_ready (self->context);
 
         register_with_display_manager (self);
+
+        register_with_systemd (self);
+}
+
+static void
+on_builtin_keybinding_triggered (MetaDisplay     *display,
+                                 MetaWindow      *window,
+                                 ClutterKeyEvent *event,
+                                 MetaKeyBinding  *binding,
+                                 KioskCompositor *self)
+{
+        g_debug ("KioskCompositor: Ignoring '%s' request",
+                 meta_key_binding_get_name (binding));
+}
+
+static void
+neuter_builtin_keybindings (KioskCompositor *self)
+{
+        const char *builtin_keybindings[] = {
+                "switch-to-workspace-1",
+                "switch-to-workspace-2",
+                "switch-to-workspace-3",
+                "switch-to-workspace-4",
+                "switch-to-workspace-5",
+                "switch-to-workspace-6",
+                "switch-to-workspace-7",
+                "switch-to-workspace-8",
+                "switch-to-workspace-9",
+                "switch-to-workspace-10",
+                "switch-to-workspace-11",
+                "switch-to-workspace-12",
+                "switch-to-workspace-left",
+                "switch-to-workspace-right",
+                "switch-to-workspace-up",
+                "switch-to-workspace-down",
+                "switch-to-workspace-last",
+                "panel-main-menu",
+                "panel-run-dialog",
+                "set-spew-mark",
+                "switch-monitor",
+                "rotate-monitor",
+                "restore-shortcuts",
+                "activate-window-menu",
+                "toggle-above",
+                "toggle-shaded",
+                "minimize",
+                "toggle-on-all-workspaces",
+                "move-to-workspace-1",
+                "move-to-workspace-2",
+                "move-to-workspace-3",
+                "move-to-workspace-4",
+                "move-to-workspace-5",
+                "move-to-workspace-6",
+                "move-to-workspace-7",
+                "move-to-workspace-8",
+                "move-to-workspace-9",
+                "move-to-workspace-10",
+                "move-to-workspace-11",
+                "move-to-workspace-12",
+                "move-to-workspace-last",
+                "move-to-workspace-left",
+                "move-to-workspace-right",
+                "move-to-workspace-up",
+                "move-to-workspace-down",
+                NULL
+        };
+        size_t i;
+
+        g_debug ("KioskCompositor: Neutering builtin keybindings");
+
+        for (i = 0; builtin_keybindings[i] != NULL; i++) {
+                meta_keybindings_set_custom_handler (builtin_keybindings[i],
+                                                     (MetaKeyHandlerFunc)
+                                                     on_builtin_keybinding_triggered,
+                                                     self,
+                                                     NULL);
+        }
 }
 
 static void
@@ -127,6 +215,7 @@ kiosk_compositor_start (MetaPlugin *plugin)
         g_autoptr (GError) error = NULL;
 
         g_set_weak_pointer (&self->display, meta_plugin_get_display (META_PLUGIN (self)));
+        g_set_weak_pointer (&self->context, meta_display_get_context (self->display));
         g_set_weak_pointer (&self->backend, clutter_get_default_backend ());
         g_set_weak_pointer (&self->stage, meta_get_stage_for_display (self->display));
 
@@ -142,10 +231,16 @@ kiosk_compositor_start (MetaPlugin *plugin)
                 g_clear_error (&error);
         }
 
+        neuter_builtin_keybindings (self);
+
         self->backgrounds = kiosk_backgrounds_new (self);
         self->input_sources_manager = kiosk_input_sources_manager_new (self);
 
-        register_session (self);
+        kiosk_gobject_utils_queue_immediate_callback (G_OBJECT (self),
+                                                      "[kiosk-compositor] register_session",
+                                                      self->cancellable,
+                                                      KIOSK_OBJECT_CALLBACK (register_session),
+                                                      NULL);
 }
 
 static void
